// SPDX-License-Identifier: GPL-2.0
/*
 * SPDX-FileCopyrightText: Copyright (c) 2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 */

#include <linux/arm-smccc.h>
#include <linux/bitfield.h>
#include <linux/bitops.h>
#include <linux/iopoll.h>
#include <linux/mmc/host.h>
#include <linux/mmc/mmc.h>
#include <linux/module.h>
#include <linux/of.h>
#include <linux/platform_device.h>
#include <linux/pm_runtime.h>
#include <linux/version.h>

#include "dw_mmc.h"
#include "dw_mmc-pltfm.h"

#define UHS_REG_EXT_SAMPLE_MASK		GENMASK(22, 16)
#define UHS_REG_EXT_DRIVE_MASK		GENMASK(29, 23)
#define BLUEFIELD_UHS_REG_EXT_SAMPLE	2
#define BLUEFIELD_UHS_REG_EXT_DRIVE	4

/* SMC call for RST_N */
#define BLUEFIELD_SMC_SET_EMMC_RST_N	0x82000007

#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 19, 0)
#define card_hw_reset hw_reset
#endif

static void dw_mci_bluefield_hw_reset(struct dw_mci *host)
{
	struct arm_smccc_res res = { 0 };

	arm_smccc_smc(BLUEFIELD_SMC_SET_EMMC_RST_N, 0, 0, 0, 0, 0, 0, 0,
		      &res);

	if (res.a0)
		pr_err("RST_N failed.\n");
}

static bool dw_mci_ctrl_reset(struct dw_mci *host, u32 reset)
{
	u32 ctrl;

	ctrl = mci_readl(host, CTRL);
	ctrl |= reset;
	mci_writel(host, CTRL, ctrl);

	/* wait till resets clear */
	if (readl_poll_timeout_atomic(host->regs + SDMMC_CTRL, ctrl,
				      !(ctrl & reset),
				      1, 500 * USEC_PER_MSEC)) {
		pr_err("eMMC timeout resetting block (ctrl reset %#x)\n",
			ctrl & reset);
		return false;
	}

	return true;
}

static void dw_mci_hw_reset(struct mmc_host *mmc)
{
	struct dw_mci_slot *slot = mmc_priv(mmc);
	struct dw_mci *host = slot->host;
	u32 bmod;

	/* Software reset of DMA */
	bmod = mci_readl(host, BMOD);
	bmod |= SDMMC_IDMAC_SWRESET;
	mci_writel(host, BMOD, bmod);

	dw_mci_ctrl_reset(host, SDMMC_CTRL_DMA_RESET |
			  SDMMC_CTRL_FIFO_RESET);

	dw_mci_bluefield_hw_reset(host);
}

/* Reuse this callback to set the card_hw_reset callback. */
static int dw_mci_bluefield_switch_voltage(struct mmc_host *mmc,
					   struct mmc_ios *ios)
{
	static struct mmc_host_ops new_ops;

	if (mmc->ops->card_hw_reset != new_ops.card_hw_reset) {
		new_ops = *mmc->ops;
		new_ops.card_hw_reset = dw_mci_hw_reset;
		mb();
		mmc->ops = (const struct mmc_host_ops *)&new_ops;
	}

	return 0;
}

static void dw_mci_bluefield_set_ios(struct dw_mci *host, struct mmc_ios *ios)
{
	u32 reg;

	/* Update the Drive and Sample fields in register UHS_REG_EXT. */
	reg = mci_readl(host, UHS_REG_EXT);
	reg &= ~UHS_REG_EXT_SAMPLE_MASK;
	reg |= FIELD_PREP(UHS_REG_EXT_SAMPLE_MASK,
			  BLUEFIELD_UHS_REG_EXT_SAMPLE);
	reg &= ~UHS_REG_EXT_DRIVE_MASK;
	reg |= FIELD_PREP(UHS_REG_EXT_DRIVE_MASK, BLUEFIELD_UHS_REG_EXT_DRIVE);
	mci_writel(host, UHS_REG_EXT, reg);
}

static const struct dw_mci_drv_data bluefield_drv_data = {
	.hw_reset		= dw_mci_bluefield_hw_reset,
	.set_ios		= dw_mci_bluefield_set_ios,
	.switch_voltage		= dw_mci_bluefield_switch_voltage
};

static const struct of_device_id dw_mci_bluefield_match[] = {
	{ .compatible = "mellanox,bluefield-dw-mshc",
	  .data = &bluefield_drv_data },
	{},
};
MODULE_DEVICE_TABLE(of, dw_mci_bluefield_match);

static int dw_mci_bluefield_probe(struct platform_device *pdev)
{
	return dw_mci_pltfm_register(pdev, &bluefield_drv_data);
}

static struct platform_driver dw_mci_bluefield_pltfm_driver = {
	.probe		= dw_mci_bluefield_probe,
	.remove		= dw_mci_pltfm_remove,
	.driver		= {
		.name		= "dwmmc_bluefield",
		.probe_type	= PROBE_PREFER_ASYNCHRONOUS,
		.of_match_table	= dw_mci_bluefield_match,
		.pm		= &dw_mci_pltfm_pmops,
	},
};

module_platform_driver(dw_mci_bluefield_pltfm_driver);

MODULE_DESCRIPTION("BlueField DW Multimedia Card driver");
MODULE_AUTHOR("Mellanox Technologies");
MODULE_LICENSE("GPL v2");
MODULE_VERSION("1.0");
